package com.smartvault.rest;

/**
 * Created on 4/16/2015.
 */

import com.smartvault.net.WebRequestInfo;
import com.smartvault.net.ClientNetContext;
import com.smartvault.proto.DelegationTokenProto.DelegationTokenRequest;
import com.smartvault.proto.DelegationTokenProto.DelegationTokenResponse;
import com.smartvault.proto.PINEndpointProto.PINEndPointRequest;
import com.smartvault.proto.PINEndpointProto.PINEndPointResponse;

public class PublicClientProtocol {
    private PublicClientCfg cfg;

    public PublicClientProtocol(PublicClientCfg cfg)
    {
        this.cfg = cfg;
    }

    private ClientNetContext createClient(String noun, String verb) throws Exception
    {
        WebRequestInfo webRequestInfo = new WebRequestInfo(this.cfg.getRootUri(), this.cfg.getClientId());
        return ClientNetContext.open(webRequestInfo, noun, verb);
    }

    public DelegatedProtocol createDelegatedProtocol( String userEmail, String userPin) throws Exception
    {
        return new DelegatedProtocol(getDelegationAccessToken(userEmail, userPin));
    }

    public UserCfg getDelegationAccessToken(String userEmail, String userPin) throws Exception
    {
        DelegationTokenRequest.PINInfo.Builder pinInfo;
        pinInfo = DelegationTokenRequest.PINInfo.newBuilder();
        pinInfo.clear();
        pinInfo.setClientId(cfg.getClientId());
        pinInfo.setPin(userPin);

        DelegationTokenRequest.Builder cbld = DelegationTokenRequest.newBuilder();
        cbld.setUserEmail(userEmail);
        cbld.setPinInfo(pinInfo);

        ClientNetContext ctx = createClient(Constants.DELEGATION_TOKEN_PATH, ClientNetContext.POST);
        ctx.writePayload(cbld.build());
        DelegationTokenResponse dtoken = ctx.readPayloadDelegationTokenResponse();

        return new UserCfg(userEmail, dtoken.getMessage().getToken(), this.cfg);
    }

    public String getPinRequestEndpoint(String userEmail) throws Exception
    {
        PINEndPointRequest.Builder cbld = PINEndPointRequest.newBuilder();
        cbld.setClientId(this.cfg.getClientId());
        ClientNetContext ctx = createClient(Constants.AUTH_URI_PATH, ClientNetContext.POST);
        ctx.writePayload(cbld.build());

        PINEndPointResponse response = ctx.readPayloadPinEndpointResponse();

        return response.getMessage().getAuthorizationUri();
    }
}
